/***************** (c) 2007 BTC Korporacja ***********************************
                     http://www kamami.com
PROJECT  : ADC
BOARD    : ZL25ARM + ZL24ARM (STR912X44)
COMPILER : GCC 4.1.1

******************************************************************************

    THE SOFTWARE INCLUDED IN THIS FILE IS FOR GUIDANCE ONLY.
    BTC KORPORACJA SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT 
    OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING 
    FROM USE OF THIS SOFTWARE.

******************************************************************************

DESCRIPTION :
    Example of AD conversion. Result of conversion is displayed on LCD
    display (HD44780 compatible).
    Jumpers settings on ZL25ARM:
	    JP5: 2-3 (ON)
        
******************************************************************************/

#include <stdio.h>

// STR9x IO register & structures
#include "91x_map.h"

// System Control Unit definitions
#include "91x_scu.h"

// Analog-to-digital converter definitions
#include "91x_adc.h"

#include "lcd.h"

// waits "howmuch" cycles
void delay(int howmuch)
{
    while(howmuch--)
        asm volatile ("nop");
}

void initialize()
{
// because GPIO ports are initially in RESET state
// we need to disable their reset signals first

// disable reset signal for GPIO port 4 and ADC
    SCU->PRR1 |= __GPIO4 | __ADC;

// enable peripheral clock for GPIO port 4 and ADC
    SCU->PCGR1 |= __GPIO4  | __ADC;

// connect line P4.0 to ADC channel 0
    SCU->GPIOIN[4] |= (1<<0);
// select P4.0 as analog input
    SCU->GPIOANA = (1<<0);

// enable ADC power
    ADC->CR |= (1<<1);
// disable threshold for channel 0
    ADC->CCR = (ADC_NoThreshold_Conversion);
// select channel 0, single mode
    ADC->CR = (1<<1) | (0<<6);
}

int adc_read()
{

// start conversion
    ADC->CR |= ADC_Conversion_Start;
// wait for end of conversion
    while (!(ADC->CR & (1<<15)));

// read ADC value
    return (ADC->DR0 & 0x3ff);
}


int main()
{

// initialize GPIO
    initialize();

// initialize LCD display
    LCD_init();
    LCD_control(ON, BLINK, SHOW);
    LCD_clear();
    LCD_setpos(0);
    LCD_putstring("ADC value: ");

    for(;;)
    {
        char buf[16];

        LCD_setpos(64);
        sprintf(buf,"%d    ", adc_read());
        LCD_putstring(buf);

        delay(50000);

    }

    return 0;
}
