#include "91x_map.h"
#include "91x_scu.h"

// LCD Port access macros

#define LCD_DATA(x) { GPIO8->DR[0x3fc] = (x); };

#define CLR_E { GPIO9->DR[0x3fc] &= ~(1<<0);  }
#define SET_E { GPIO9->DR[0x3fc] |= (1<<0);  }

#define CLR_RS { GPIO9->DR[0x3fc] &= ~(1<<2);  }
#define SET_RS { GPIO9->DR[0x3fc] |= (1<<2);  }

#define CLR_RW { GPIO9->DR[0x3fc] &= ~(1<<1);  }
#define SET_RW { GPIO9->DR[0x3fc] |= (1<<1);  }

static void LCD_delay(int cycles) 
{
    cycles*=50;
    while(cycles--) asm volatile ("nop");
}

static void LCD_write(unsigned char value, int command)
{
    CLR_RW;
    if(command) CLR_RS else SET_RS;
		
#ifdef LCD_4BIT
    LCD_DATA((value)&0xf0);
    SET_E;
    LCD_delay(2);
    CLR_E;
    LCD_DATA((value<<4)&0xf0);
    SET_E;
    LCD_delay(2);
    CLR_E;   
#else
    LCD_DATA(value);
    SET_E;
    LCD_delay(1);
    CLR_E;
#endif	
	
    LCD_delay(4);
}

void LCD_init()
{
// disable reset signal for GPIO ports 8 and 9
    SCU->PRR1 |= __GPIO8 | __GPIO9;

// enable peripheral clock for GPIO ports 8 and 9
    SCU->PCGR1 |= __GPIO8 | __GPIO9;

// set output type to push-pull for all port 8 lines and lines 0-2 for port 9
    SCU->GPIOTYPE[8] = 0x0000 ;
    SCU->GPIOTYPE[9] &= ~0x3f ;

// set all port 8 lines to output (both registers below need to be set for GPIO to work)
    SCU->GPIOOUT[8]  = 0x5555;
    GPIO8->DDR = 0xff;

// set port 9 lines 0..2 to output
    SCU->GPIOOUT[9]  &= ~0x3f ;
    SCU->GPIOOUT[9]  |= 0x15 ;
    GPIO9->DDR |= 0x7;

    LCD_write(0x30, 1); LCD_delay(200);
    LCD_write(0x30, 1); LCD_delay(200);
    LCD_write(0x30, 1); LCD_delay(200);

    LCD_write(0x38, 1); 
    LCD_write(0x01, 1); 
    LCD_write(0x06, 1); 
    LCD_write(0x0c, 1); 
    LCD_write(0x02, 1); 
    LCD_write(0x80, 1); 

    LCD_delay(100);	
}

void LCD_clear()
{
    LCD_write(1, 1);
    LCD_delay(100);	
}

void LCD_putchar(unsigned char value)
{
    LCD_write(value,0);
}

void LCD_putstring(char *str)
{
    while(*str) 
        LCD_putchar(*str++);
}

void LCD_control(unsigned char dsp,unsigned char blink,unsigned char cursor)
{
    unsigned char control;  // variable to generate instruction byte

    control = (0x08 + blink + cursor + dsp); // Cursor control

    LCD_write(control, 1);
    LCD_delay(100);	
}

void LCD_setpos(unsigned char p)
{
    LCD_write(0x80 | p, 1);
    LCD_delay(10);	
}
