/***************** (c) 2007 BTC Korporacja ***********************************
                     http://www kamami.com
PROJECT  : UART
BOARD    : ZL25ARM + ZL24ARM (STR912X44)
COMPILER : GCC 4.1.1

******************************************************************************

    THE SOFTWARE INCLUDED IN THIS FILE IS FOR GUIDANCE ONLY.
    BTC KORPORACJA SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT 
    OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING 
    FROM USE OF THIS SOFTWARE.

******************************************************************************

DESCRIPTION :
    Simple UART communication demo.
    Jumpers settings on ZL25ARM:
	    not required
	    RS232 cable should be connected to ZL4 connector
        
******************************************************************************/

// STR9x IO register & structures
#include "91x_map.h"

// System Control Unit definitions
#include "91x_scu.h"

// UART definitions
#include "91x_uart.h"

// CPU clock frequency in Hz
#define MAIN_CLOCK 20000000

// UART speed
#define UART_SPEED 38400


// UART initialization stuff.
void initialize_uart()
{
// because GPIO ports and UART are initially in RESET state
// we need to disable their reset signals first

// disable reset signal for GPIO ports 3, 5 and UART
    SCU->PRR1 |= __GPIO5 | __GPIO3 | __UART1;

// enable peripheral clock for GPIO ports 3, 5 and UART
    SCU->PCGR1 |= __GPIO5 | __GPIO3 | __UART1;

// Output alternate 3 for pin P3.3 (UART_TX)
    SCU->GPIOOUT[3] = (2<<(3*2));

// Set "IP Connected" bit for pin P3.2 (UART_RX) to connect it
// to UART peripheral
    SCU->GPIOIN[3] = (1<<2);

// Set P3.3 pin (UART_TX) direction to output
    GPIO3->DDR = (3<<1);

// Calculate baudrate divider. By default UART bitrate clock is main clock divided
// by 2.

    int ipart = 64 * (MAIN_CLOCK / 2) / (16 * UART_SPEED);
    int fpart = ipart & 0x3f; 

// load integer and fractional part of UART baudrate divider into IBRD and FBRD registers
    UART1->IBRD = ipart >> 6;
    UART1->FBRD = fpart;

// set frame parameters: 8 data bits, 1 stop bit, no parity
    UART1->LCR = UART_WordLength_8D;

// set transmin and receive modes and enable UART.
    UART1->CR = UART_Mode_Tx_Rx | 1;
}

// sends single byte
void uart_write_byte(unsigned char c)
{
// wait for transmit FIFO empty
    while(UART1->FR & (1<<5));

// write byte to UART data register
    UART1->DR = c;
}    

// receives single byte
char uart_read_byte()
{
// wait for incoming byte (FIFO status bit)
    while(!(UART1->FR & (1<<6))); 

// read it from UART data register 
    return UART1->DR & 0xff;
}

// transmits ASCIIZ string via UART
void uart_write_string(char *str)
{
    char c;
    
    while(c = *str++) uart_write_byte(c);
}


int main()
{

// initialize the UART ;)
    initialize_uart();

    for(;;)
    {
        char buf[1024], c;
        int buf_pos = 0;
	
        uart_write_string("\n\r\n\rHello! This is BTC STR912 Board UART Test Program. If you can read this text, UART works!\n\r");
        uart_write_string("\n\rWhat's your name? ");

        // receive name, terminated by '\r'	
        while(((c = uart_read_byte()) != '\r') && buf_pos<1023) { buf[buf_pos++] = c;uart_write_byte(c); }
	
        buf[buf_pos] = 0;
	
        uart_write_string("\n\rNice to meet you, ");
        uart_write_string(buf);
        uart_write_string(".\n\r\n\r");
    }
    return 0;
}
